#include <GL/glew.h>
#include <stdlib.h>
#include <time.h>
#include <SDL.h>
#include <QImage.h>
#include <qapplication.h>
#include <QDom.h>
#include <QFile.h>
#include <QGL.h>
#include <QdateTime.h>
#include <QDir.h>

#include <vector>

#include <vcg/Point3.h>
#include <vcg/Matrix44.h>

using namespace vcg;
using namespace std;

#include <CI/CISG.h>
#include <CI/CIMd2.h>
#include <MM/MMSG.h>

#include "../bmf/BMF_font.h"

enum {MDGame,MDIntro,MDLevel,MDWinLose,MDEnd};

QTime tm;
MMSG sg;

BMF_Font *bfont;
float spx,spy,spz,camx,camy,camz;
const int winWidth=800;
const int winHeight=600;
const int MAX_LEV=16;

class SDLButton
{
public:
  Point2i p1,p2;
  QString text;
  GLuint  texid;
  SDLButton() {}
  SDLButton(Point2i &_p1,Point2i &_p2,QString _text): p1(_p1),p2(_p2),text(_text),texid(0) {}
  SDLButton(Point2i &_p1,Point2i &_p2,QString _text,GLuint _id): p1(_p1),p2(_p2),text(_text),texid(_id){}

  void Draw()
  {
    glColor3f(.3f, .3f, .3f);
	  glBegin(GL_QUADS);
	    glVertex2f(p1[0],p1[1]);
	    glVertex2f(p2[0],p1[1]);
	    glVertex2f(p2[0],p2[1]);
	    glVertex2f(p1[0],p2[1]);
	  glEnd();
    float w= BMF_GetTextWidth(bfont,text);
	  BMF_Begin(bfont);
	  BMF_SetPos((p1[0]+p2[0])/2-w/2,(p1[1]+p2[1])/2);
	  BMF_Print(text);
	  BMF_End();
  };

 void DrawTextured()
  {
    glColor3f(1,1,1);
    glBindTexture(GL_TEXTURE_2D, texid);
	  glBegin(GL_QUADS);
	    glTexCoord2f(0.0f,0.0f); glVertex2f(p1[0],p1[1]);
	    glTexCoord2f(1.0f,0.0f); glVertex2f(p2[0],p1[1]);
	    glTexCoord2f(1.0f,1.0f); glVertex2f(p2[0],p2[1]);
	    glTexCoord2f(0.0f,1.0f); glVertex2f(p1[0],p2[1]);
	  glEnd();
    
    float w= BMF_GetTextWidth(bfont,text);
	  BMF_Begin(bfont);
	  BMF_SetPos((p1[0]+p2[0])/2-w/2,p2[1] + 5);
	  BMF_Print(text);
	  BMF_End();
  };

  bool Picked(Point2i p)
  {
    p[1]=winHeight-p[1];
    if(p[0]>p1[0] && p[0]<p2[0] && p[1]>p1[1] && p[1]<p2[1] ) return true;
      return false;
  }
};

SDLButton IntroQuitButton(Point2i(100,100),Point2i(200,150),"quit");
SDLButton IntroLoadButton(Point2i(500,100),Point2i(600,150),"load");
SDLButton WinLoseButton(Point2i(300,100),Point2i(600,150),"Return to main menu");
SDLButton LevelIcons[MAX_LEV];
QString livelli[MAX_LEV];
int contfn; 

bool animBall=false;

void Enter2D(void) {
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(0, winWidth, 0, winHeight, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}

void Exit2D(void) {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

	

void DrawGLScene()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	float spec[4]={.4f,.4f,.4f,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	float lightpos[4]={0,0,1,0};
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//glFrustum(-0.2f,0.2f,-0.15f,0.15f,0.2f,20.0f);
	gluPerspective(60,float(winWidth)/winHeight,0.2f,20.0f);
  glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();
	glLightfv(GL_LIGHT0,GL_POSITION,lightpos);

	gluLookAt(spx*sg.g->scalefactor-1.0f+sg.g->viewzoom*sin(sg.g->viewangle),spz*sg.g->scalefactor+sg.g->viewaltitude,-spy*sg.g->scalefactor+1.0f+sg.g->viewzoom*cos(sg.g->viewangle),spx*sg.g->scalefactor-1.0f,spz*sg.g->scalefactor,-spy*sg.g->scalefactor+1.0f,0,1,0);

	glColor3f(1,.4f,.4f);
	sg.drawGL(clock()/1000.0f);	
  
  Enter2D();
  	BMF_Begin(bfont);
		BMF_SetPos(320,420);
		BMF_Print("TIME %i",sg.g->leveltime);

		if (sg.g->pause) {
			BMF_SetPos(winWidth/2,winHeight/2);
			BMF_Print("PAUSED");
		}
	  BMF_End();
  Exit2D();
	SDL_GL_SwapBuffers();
}

void DrawIntro() {
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		Enter2D();
		
	BMF_Begin(bfont);
	BMF_SetPos(60,550);
	BMF_SetScale(2.5f,2.0f);
	BMF_SetColor(1.0f,0.9f,0.9f,1.0f);
	BMF_Print("Marble Madness 3D 2005");

	BMF_SetScale(1.5f,1);
	//BMF_SetColor(0.9,0.9,1,1);
	BMF_SetPos(50,500);
	BMF_Print("Credits:");
	
  int xpos=50,ypos=430,line=20;
	BMF_SetColor(0.9f,0.9f,1.0f,1.0f);
	BMF_SetPos(xpos,ypos-=line);
	BMF_Print("Codice iniziale di Paolo Cignoni");
	BMF_SetPos(xpos,ypos-=line);
	BMF_Print("Modificato secondo le specifiche dell'esame da:");
  BMF_SetPos(xpos,ypos-=line);
	BMF_SetPos(xpos,ypos-=line);
	BMF_Print("Andrea Boscato");
	BMF_SetPos(xpos,ypos-=line);
	BMF_Print("Domenico Arenga");
	BMF_SetPos(xpos,ypos-=line);
	BMF_Print("Alessandro Manzi");
	BMF_SetPos(xpos,ypos-=line);
	BMF_SetPos(xpos,ypos-=line);
	BMF_Print("(Ulteriore ripulitura di Paolo Cignoni)");
	
	BMF_End();
  IntroQuitButton.Draw();
  IntroLoadButton.Draw();


	Exit2D();
	SDL_GL_SwapBuffers();
}

void DrawWin() {

	
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	Enter2D();
	
	glDisable(GL_TEXTURE_2D|GL_COLOR_MATERIAL);
	
	glColor3f(1.0f,0,0);

	glBegin(GL_QUADS);
	glVertex2f(50,50);
	glVertex2f(150,50);
	glVertex2f(150,100);
	glVertex2f(50,100);

	glEnd();

	BMF_Begin(bfont);
	BMF_SetPos(60,400);
	BMF_SetScale(2.5,2);
	BMF_SetColor(1,0.9f,0.9f,1);
	BMF_Print("You Win!!!!");

	BMF_SetScale(1.5,1);
	//BMF_SetColor(0.9,0.9,1,1);
	BMF_SetPos(50,350);
	BMF_Print("Congratulations!");


	BMF_End();
	WinLoseButton.Draw();
			
	Exit2D();

	SDL_GL_SwapBuffers();
}


void DrawLose() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	Enter2D();
	
	glColor3f(1.0f,0,0);
	glBegin(GL_QUADS);
	  glVertex2f(50,50);
	  glVertex2f(150,50);
	  glVertex2f(150,100);
	  glVertex2f(50,100);
	glEnd();

	BMF_Begin(bfont);
	BMF_SetPos(60,400);
	BMF_SetScale(2.5,2);
	BMF_SetColor(1,0.9f,0.9f,1);
	BMF_Print("You Lose!!!!");

	BMF_SetScale(1.5,1);
	//BMF_SetColor(0.9,0.9,1,1);
	BMF_SetPos(50,350);
	BMF_Print("GAME OVER");


	BMF_End();
	
WinLoseButton.Draw();
	
	Exit2D();
  		
	SDL_GL_SwapBuffers();
}
void DrawLevels() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		Enter2D();
	static bool firstTime=true;
	static GLuint thumb[MAX_LEV];
  BMF_Begin(bfont);

	BMF_SetPos(80,420);
	BMF_SetScale(2.5,2);
	BMF_SetColor(0.9f,0.4f,0.65f,1);
	BMF_Print("LEVELS");

	BMF_End();
	

	QDir d("levels");
	d.setFilter(QDir::Files);
	d.setNameFilter("*.xml");
	const QFileInfoList *list = d.entryInfoList();
	QFileInfoListIterator it(*list);
	QFileInfo *fi;

	
	contfn=0;
	while((fi = it.current()) != 0 && contfn< MAX_LEV) {
		
        livelli[contfn]= fi->fileName();
        contfn++;
		++it;
	}

	glColor3f(1,1,1);
	glEnable(GL_TEXTURE_2D);
	QImage img; 

  Point2i Offset1(-50,-50);
  Point2i Offset2( 50, 50);
  int spacing = 200;
  Point2i BasePos(100,500);
  Point2i CurPos(BasePos);
if(firstTime) glGenTextures(MAX_LEV, thumb);
	    
	for(int i=0; i<contfn; i++) {
	  if(firstTime)
    {
      img.load("levels/"+livelli[i]+".png");
		  QImage txthumb = QGLWidget::convertToGLFormat(img);
		  glBindTexture(GL_TEXTURE_2D, thumb[i]);
		  gluBuild2DMipmaps(GL_TEXTURE_2D, 3, txthumb.width(), txthumb.height(), GL_RGBA, GL_UNSIGNED_BYTE, txthumb.bits() );
      LevelIcons[i]=SDLButton(CurPos+Offset1,CurPos+Offset2,livelli[i],thumb[i]);
    }

    LevelIcons[i].DrawTextured();

    CurPos[0]+=spacing;
    if(CurPos[0]>winWidth) {
      CurPos[0]=BasePos[0];
      CurPos[1]-=spacing;
    }
  }

	Exit2D();
	SDL_GL_SwapBuffers();
  firstTime=false;
}

// Inizializzazione dello stato di OpenGL
void myGLInit()
{
  glewInit();
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);

	// abilita i texture environment necessari
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

  printf("loading font\n");
  bfont = BMF_Load("bmf/data/fnt1.bmf");
  if(!bfont) 
  {
    printf("Failed to load font. Started in the wrong dir?\n");
    exit(-1);
  }
  sg.root.initializeGL();

}

void myGLReshapeFunc(GLsizei w, GLsizei h)
{
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
	myGLInit();
}



int LoopIntro()
{
	while ( true ) {
		SDL_Event event;
		int ret=SDL_PollEvent(&event);
		if(ret) {
			switch(event.type) {  
				case SDL_QUIT    : 
				case SDL_KEYDOWN :return MDEnd;
				case SDL_KEYUP :
                    switch(event.key.keysym.sym ) {
                        case ' ' :	return MDLevel;
                    }
                    break;

				case SDL_VIDEORESIZE	:
					myGLReshapeFunc(event.resize.w,event.resize.h); 
					break;
				case SDL_MOUSEBUTTONDOWN :
					if (IntroQuitButton.Picked(Point2i(event.button.x,event.button.y))) return MDEnd;
					if (IntroLoadButton.Picked(Point2i(event.button.x,event.button.y))) return MDLevel;
    			break;
			}  
		} else 		DrawIntro();
	}
}

 int LoopWinLose()
 {
int    done=0;
	while ( !done ) {
		SDL_Event event;
int		ret=SDL_PollEvent(&event);
		if(ret) {
			switch(event.type) {  
				case SDL_QUIT    : 					return MDEnd;               
				case SDL_KEYDOWN :
					if ( event.key.keysym.sym == SDLK_ESCAPE ) done = 1;
					break;
				case SDL_VIDEORESIZE	:
					myGLReshapeFunc(event.resize.w,event.resize.h); 
					break;
				case SDL_MOUSEBUTTONDOWN : 
          if(WinLoseButton.Picked(Point2i(event.button.x,event.button.y)) ) done=1;
					break;
			}  
		} else {
			if (sg.g->win) 	DrawWin();
      else    			 DrawLose();
      }
		}
    return MDIntro;
	}



 int LoopChooseLevel()
 {
/****  Schermata load *****/
	int load = false;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	int livelloDaKar;
	int done=0; 
	while((!load) && (!done)) {
		
	  SDL_Event event;
		int ret=SDL_PollEvent(&event);
		if(ret) {
			switch(event.type) {  
				case SDL_QUIT    : return MDEnd;
				case SDL_KEYDOWN :
					if ( event.key.keysym.sym == SDLK_ESCAPE ) return MDIntro;
					break;
				case SDL_VIDEORESIZE	:
					myGLReshapeFunc(event.resize.w,event.resize.h); 
					break;

				case SDL_MOUSEBUTTONDOWN :
          for(int i=0;i<contfn;++i)
          {
            if(LevelIcons[i].Picked(Point2i(event.button.x,event.button.y) ))
            {	
              livelloDaKar = i;
						  load=1;	
					  }
          }
					
      }
    } else DrawLevels();
  }

  sg.XMLRead("levels/"+livelli[livelloDaKar]);
	sg.g->setTexture("data/tile.png");

  return MDGame;
}




int LoopGame()
{
   int done=0;
  sg.g->setBallOnStart();
	//sg.g->generateFace();
	animBall = true;
	//GLView->animBall=!GLView->animBall;
 
 /* Loop, drawing and checking events */

 myGLReshapeFunc(winWidth,winHeight);  
 int moved=false;
 int currenttime=0;
 if (!done) {
 sg.g->viewzoom=2+0.01f;
 sg.g->viewangle=0;
 sg.g->viewaltitude=2;
 sg.g->win=0;
 sg.g->lose=0;
 sg.g->pause=0;
 currenttime=sg.g->leveltime;
 }
 float currentclock=clock();
int sfff=0;
float savedclock=0;

 while ( (!done) && (!sg.g->win) && (!sg.g->lose)) {
	int mousex;//= 320;
	int mousey;// 240;
	
    SDL_Event event;
	int ret=SDL_PollEvent(&event);
	if(ret)
	   switch(event.type) {  
			case SDL_QUIT    : done = 1;  
							   break ;
			case SDL_KEYDOWN : if ( event.key.keysym.sym == SDLK_ESCAPE ) 
							        done = 1;
									break;
            case SDL_KEYUP : {
                        switch(event.key.keysym.sym ) {
                                case ' ' :
                                        printf("Started Animation");
                                        sg.g->setBallOnSelectionCenter();
                                        sg.g->generateFace();
                                        animBall=!animBall;
										break;
								case SDLK_LEFT :
									sg.g->viewangle-=M_PI/18.0f;
									break;
								case SDLK_RIGHT :
									sg.g->viewangle+=M_PI/18.0f;
									break;
								case SDLK_PAGEUP :
									sg.g->viewzoom+=0.5f;
									break;
								case SDLK_PAGEDOWN :
									if(sg.g->viewzoom>0.7f) sg.g->viewzoom-=0.5f;
									break;
								case SDLK_UP :
									sg.g->viewaltitude+=0.5f;
									break;
								case SDLK_DOWN :
									sg.g->viewaltitude-=0.5f;
									break;
								case 'p' :
									sg.g->pause = !sg.g->pause;
									break;
                                }
                                break;
                             }
			case SDL_VIDEORESIZE	: //SDL_SetVideoMode(event.resize.w,event.resize.h, 0, SDL_OPENGL | SDL_RESIZABLE);
									  myGLReshapeFunc(event.resize.w,event.resize.h); 
									break;

			case SDL_MOUSEBUTTONDOWN :
					mousex=event.button.x;
					mousey=event.button.y;
					moved=1;
					break;

			case SDL_MOUSEBUTTONUP :
				if (moved) {
					int mouse2x = event.button.x - mousex;
					int mouse2y = -event.button.y + mousey;
					if (!sg.g->pause) {
						sg.g->moveSphere((cos(sg.g->viewangle)*mouse2x/2.0f-sin(sg.g->viewangle)*mouse2y/2.0f),(+sin(sg.g->viewangle)*mouse2x/2.0f+cos(sg.g->viewangle)*mouse2y/2.0f));
					
						animBall = true;
						moved=false;
					}
				} 

	   }  else {
			if(animBall) sg.g->updateSpherePos(tm.elapsed()/1000.f);
			Point3f sppos=sg.g->getSpherePosition();
			Point3f spvect=sg.g->getSphereMovement().Normalize();

			spx=sppos[0];			spy=sppos[1];			spz=sppos[2];
			//Point3f spvect=sg.g->sp.v.Normalize();
			camx=spx-3.0f;//*spvect[0];
			camy=spy-3.0f;//*spvect[1];
			camz=spz-3.0f;//*spvect[2];

			if(sg.g->checkFall() < -5)
			{
				sg.g->lose=1;
			}

			if (!sg.g->pause) {
				if (sfff) {
					currentclock=clock()-savedclock;
					sfff=0;
				}
				sg.g->leveltime=currenttime-(-currentclock+clock())/1000.0f;
			} else {
				if (!sfff) {
                    sfff=1;
					savedclock=clock()-currentclock;
				}
			}
			if (sg.g->leveltime<1) {
				sg.g->lose=1;
			}
			DrawGLScene();
	   }
 }

 if(sg.g->win || sg.g->lose) return MDWinLose; 
return MDIntro;
}












int main(int argc, char **argv) {  
	printf("Started \n");

	/* Initialize SDL for video output */
	 if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		exit(1);
	 }
  /* Specify the requirements for the gl window*/
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 32);   
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);   
  
	/* Create a 640x480 OpenGL screen */
//SDL_Surface* sdls;
  if ( SDL_SetVideoMode(winWidth, winHeight, 32, SDL_OPENGL ) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }


/**** schermata di presentazione ****/
	myGLReshapeFunc(winWidth,winHeight);  

int mode=MDIntro;
 while (mode != MDEnd)
{
    if(mode==MDGame)  mode= LoopGame();
    if(mode==MDIntro) mode= LoopIntro();
    if(mode==MDLevel) mode= LoopChooseLevel();
    if(mode==MDWinLose)  mode= LoopWinLose();
}

  /* Cleaning up */
	// BMF_Free(bfont);
  SDL_Quit();
  return 1;
}





